from __future__ import annotations
import glob
import os
import shutil
import typing

from bcps import core
import re


class Path:
    def __init__(self, path: str = "", is_relative: bool = False):
        if is_relative:
            self.path = self.get_relative_path(path)
        else:
            self.path = path

    def is_relative(self) -> bool:
        return not os.path.isabs(self.path)

    @staticmethod
    def get_root() -> Path:
        return Path(os.sep)

    def get_relative_path(self, path: str) -> str:
        return os.path.join(self.get_files_folder().path, path)

    @staticmethod
    def get_files_folder() -> Path:
        file = Path(os.path.realpath(__file__))
        if file.get_extension() == "pyc":
            path = file.parent().parent().parent().parent().add("files")
        else:
            path = file.parent().parent().parent().add("files")
        return path

    def strip_trailing_slash(self) -> Path:
        return Path(self.path.rstrip("/"))

    def open(self):
        self.generate_dirs()
        if os.name == "nt":
            os.startfile(self.path)  # type: ignore
        elif os.name == "posix":
            cmd = f"dbus-send --session --dest=org.freedesktop.FileManager1 --type=method_call --print-reply /org/freedesktop/FileManager1 org.freedesktop.FileManager1.ShowItems array:string:'file://{self.path}' string:''"
            core.Command(cmd, display_output=False).run_in_thread()
        elif os.name == "mac":
            core.Command(f"open {self.path}", display_output=False).run()
        else:
            raise OSError("Unknown OS")

    def open_file(self):
        os_name = os.name
        if os_name == "nt":
            os.startfile(self.path)  # type: ignore
        elif os_name == "posix":
            cmd = f"xdg-open {self.path}"
            core.Command(cmd, display_output=False).run_in_thread()
        elif os_name == "mac":
            core.Command(f"open {self.path}", display_output=False).run()
        else:
            raise OSError("Unknown OS")

    def run(self, arg: str = "", display_output: bool = False) -> core.CommandResult:
        cmd_text = self.path + " " + arg
        cmd = core.Command(cmd_text, display_output=display_output)
        return cmd.run()

    def to_str(self) -> str:
        return self.path

    def to_str_forwards(self) -> str:
        return self.path.replace("\\", "/")

    @staticmethod
    def get_documents_folder(app_name: str = "bcps") -> Path:
        os_name = os.name
        if os_name == "nt":
            path = Path.join(os.environ["USERPROFILE"], "Documents", app_name)
        elif os_name == "posix":
            path = Path.join(os.environ["HOME"], "Documents", app_name)
        elif os_name == "mac":
            path = Path.join(os.environ["HOME"], "Documents", app_name)
        else:
            raise OSError("Unknown OS")
        path.generate_dirs()
        return path

    def generate_dirs(self: Path) -> Path:
        if not self.exists():
            try:
                self.__make_dirs()
            except OSError:
                pass
        return self

    def create(self) -> Path:
        if not self.exists():
            self.write(core.Data("test"))
        return self

    def exists(self) -> bool:
        return os.path.exists(self.path)

    def __make_dirs(self) -> Path:
        os.makedirs(self.path)
        return self

    def basename(self) -> str:
        return os.path.basename(self.path)

    @staticmethod
    def join(*paths: str | Path) -> Path:
        _paths: list[str] = [str(path) for path in paths]
        return Path(os.path.join(*_paths))

    @typing.overload
    def add(self, *paths: Path) -> Path: ...

    @typing.overload
    def add(self, *paths: str) -> Path: ...

    def add(self, *paths: str | Path) -> Path:
        _paths: list[str] = [str(path) for path in paths]
        return Path(os.path.join(self.path, *_paths))

    def strip_leading_slash(self) -> Path:
        return Path(self.path.lstrip("/").lstrip("\\"))

    def __str__(self) -> str:
        return self.path

    def __repr__(self) -> str:
        return self.path

    def remove_tree(self, ignoreErrors: bool = False) -> Path:
        if self.exists():
            shutil.rmtree(self.path, ignore_errors=ignoreErrors)
        return self

    def remove(self):
        if self.exists():
            if self.is_directory():
                self.remove_tree()
            else:
                os.remove(self.path)

    def has_files(self) -> bool:
        return len(os.listdir(self.path)) > 0

    def copy(self, target: Path):
        if self.exists():
            if self.is_directory():
                self.copy_tree(target)
            else:
                try:
                    shutil.copy(self.path, target.path)
                except shutil.SameFileError:
                    pass
        else:
            raise FileNotFoundError(f"File not found: {self.path}")

    def copy_thread(self, target: Path):
        core.Thread("copy", self.copy, (target,)).start()

    def copy_tree(self, target: Path):
        if target.exists():
            target.remove_tree()
        if self.exists():
            shutil.copytree(self.path, target.path)

    def read(self, create: bool = False) -> core.Data:
        if self.exists():
            return core.Data.from_file(self)
        elif create:
            self.write(core.Data())
            return self.read()
        else:
            raise FileNotFoundError(f"File not found: {self.path}")

    def write(self, data: core.Data):
        data.to_file(self)

    def get_paths_dir(self, regex: str | None = None) -> list[Path]:
        if self.exists():
            if regex is None:
                return [self.add(file) for file in os.listdir(self.path)]
            else:
                files: list[Path] = []
                for file in os.listdir(self.path):
                    if re.search(regex, file):
                        files.append(self.add(file))
                return files
        return []

    def get_files(self, regex: str | None = None) -> list[Path]:
        return [file for file in self.get_paths_dir(regex) if file.is_file()]

    def is_file(self) -> bool:
        return os.path.isfile(self.path)

    def get_dirs(self) -> list["Path"]:
        return [file for file in self.get_paths_dir() if file.is_directory()]

    def glob(self, pattern: str, recursive: bool = False) -> list[Path]:
        return [
            Path(path)
            for path in glob.glob(self.add(pattern).path, recursive=recursive)
        ]

    def strip_path_from(self, path: Path) -> Path:
        return Path(self.path.replace(path.path, "")).strip_leading_slash()

    def is_directory(self) -> bool:
        return os.path.isdir(self.path)

    def change_name(self, name: str) -> Path:
        return self.parent().add(name)

    def rename(self, name: str, overwrite: bool = False):
        if not self.exists():
            raise FileNotFoundError(f"File not found: {self.path}")
        new_path = self.change_name(name)
        if new_path.path == self.path:
            return
        if new_path.exists():
            if overwrite:
                new_path.remove()
            else:
                raise FileExistsError(f"File already exists: {new_path}")
        os.rename(self.path, new_path.path)
        self.path = new_path.path

    def parent(self) -> Path:
        return Path(os.path.dirname(self.path))

    def change_extension(self, extension: str) -> Path:
        if extension.startswith("."):
            extension = extension[1:]
        return Path(self.path.rsplit(".", 1)[0] + "." + extension)

    def remove_extension(self) -> Path:
        return Path(self.path.rsplit(".", 1)[0])

    def get_extension(self) -> str:
        try:
            return self.path.rsplit(".", 1)[1]
        except IndexError:
            return ""

    def get_file_name(self) -> str:
        return os.path.basename(self.path)

    def get_file_name_path(self) -> Path:
        return Path(self.get_file_name())

    def get_file_name_without_extension(self) -> str:
        return self.get_file_name().rsplit(".", 1)[0]

    def get_file_size(self) -> int:
        return os.path.getsize(self.path)

    def get_absolute_path(self) -> Path:
        return Path(os.path.abspath(self.path))

    def copy_object(self) -> Path:
        return Path(self.path)



from bcps.cli import (
    color,
    dialog_creator,
    main,
    file_dialog,
    feature_handler,
    save_management,
    server_cli,
    edits,
)

__all__ = [
    "color",
    "dialog_creator",
    "main",
    "file_dialog",
    "feature_handler",
    "save_management",
    "server_cli",
    "edits",
]



from __future__ import annotations
import dataclasses
import tempfile
from typing import Any
from bcps import core
from bcps.cli import color


class ThemeHandler:
    def __init__(self, theme_code: str | None = None):
        if theme_code is None:
            self.theme_code = core.core_data.config.get_str(core.ConfigKey.THEME)
        else:
            self.theme_code = theme_code

        self.theme_data = self.get_theme_data()

    @staticmethod
    def get_themes_folder() -> core.Path:
        return core.Path("themes", True).generate_dirs()

    @staticmethod
    def get_external_themes_folder() -> core.Path:
        return core.Path.get_documents_folder().add("external_themes").generate_dirs()

    @staticmethod
    def get_theme_path(theme_code: str) -> core.Path:
        if theme_code.startswith("ext-"):
            return ThemeHandler.get_external_themes_folder().add(theme_code + ".json")
        return ThemeHandler.get_themes_folder().add(theme_code + ".json")

    def get_theme_data(self) -> dict[str, Any]:
        file_path = self.get_theme_path(self.theme_code)
        if not file_path.exists():
            return {}
        try:
            return core.JsonFile.from_data(file_path.read()).to_object()
        except core.JSONDecodeError:
            return {}

    def get_short_name(self) -> str:
        return self.theme_data.get("short_name", "")

    def get_name(self) -> str:
        return self.theme_data.get("name", "")

    def get_description(self) -> str:
        return self.theme_data.get("description", "")

    def get_author(self) -> str:
        return self.theme_data.get("author", "")

    def get_version(self) -> str:
        return self.theme_data.get("version", "")

    def get_git_repo(self) -> str | None:
        return self.theme_data.get("git_repo", None)

    def get_theme_colors(self) -> dict[str, Any]:
        return self.theme_data.get("colors", {})

    def get_theme_color(self, color_code: str) -> str:
        return self.get_theme_colors().get(color_code, "")

    def get_primary_color(self) -> str:
        return self.get_theme_color("primary")

    def get_secondary_color(self) -> str:
        return self.get_theme_color("secondary")

    def get_tertiary_color(self) -> str:
        return self.get_theme_color("tertiary")

    def get_quaternary_color(self) -> str:
        return self.get_theme_color("quaternary")

    def get_error_color(self) -> str:
        return self.get_theme_color("error")

    def get_warning_color(self) -> str:
        return self.get_theme_color("warning")

    def get_success_color(self) -> str:
        return self.get_theme_color("success")

    @staticmethod
    def get_all_themes() -> list[str]:
        themes = [
            file.get_file_name_without_extension()
            for file in ThemeHandler.get_themes_folder().get_paths_dir(
                regex=r".*\.json"
            )
        ]
        themes += [
            folder.get_file_name_without_extension()
            for folder in ThemeHandler.get_external_themes_folder().get_paths_dir(
                regex=r".*\.json"
            )
        ]
        return themes

    @staticmethod
    def remove_theme(theme_code: str):
        extern = ExternalThemeManager.get_external_theme(theme_code)
        if extern is not None:
            ExternalThemeManager.delete_theme(extern)

        ThemeHandler.get_theme_path(theme_code).remove()
        if theme_code == core.core_data.config.get_str(core.ConfigKey.THEME):
            core.core_data.config.set_default(core.ConfigKey.THEME)


@dataclasses.dataclass
class ExternalTheme:
    short_name: str
    name: str
    description: str
    author: str
    version: str
    colors: dict[str, Any]
    git_repo: str | None = None

    def to_json(self) -> dict[str, Any]:
        return dataclasses.asdict(self)

    @staticmethod
    def from_json(json_data: dict[str, Any]) -> ExternalTheme | None:
        try:
            return ExternalTheme(**json_data)
        except TypeError:
            return None

    @staticmethod
    def from_git_repo(git_repo: str) -> ExternalTheme | None:
        repo = core.GitHandler().get_repo(git_repo)
        if repo is None:
            return None
        theme_json = repo.get_file(core.Path("theme.json"))
        if theme_json is None:
            return None
        json_data = core.JsonFile.from_data(theme_json).to_object()
        json_data["git_repo"] = git_repo
        return ExternalTheme.from_json(json_data)

    def get_new_version(self) -> bool:
        if self.git_repo is None:
            return False
        repo = core.GitHandler().get_repo(self.git_repo)
        if repo is None:
            return False
        with tempfile.TemporaryDirectory() as tmp:
            temp_dir = core.Path(tmp)
            success = repo.clone_to_temp(temp_dir)
            if not success:
                return False
            external_theme = ExternalThemeManager.parse_external_theme(
                temp_dir.add("theme.json")
            )
            if external_theme is None:
                return False
            version = external_theme.version

            if version == self.version:
                return False

            self.name = external_theme.name
            self.short_name = external_theme.short_name
            self.description = external_theme.description
            self.author = external_theme.author
            self.colors = external_theme.colors
            self.version = version

        success = repo.pull()
        if not success:
            return False
        self.save()
        return True

    def save(self):
        ExternalThemeManager.save_theme(self)

    def get_full_name(self) -> str:
        return f"ext-{self.author}-{self.short_name}"


class ExternalThemeManager:
    @staticmethod
    def delete_theme(external_theme: ExternalTheme):
        if external_theme.git_repo is None:
            return
        folder = core.GitHandler.get_repo_folder().add(
            external_theme.git_repo.split("/")[-1]
        )
        folder.remove()

    @staticmethod
    def save_theme(
        external_theme: ExternalTheme,
    ):
        """Saves an external theme.

        Args:
            external_theme (ExternalTheme): External theme to save.
        """
        if external_theme.git_repo is None:
            return
        file = ThemeHandler.get_theme_path(external_theme.get_full_name())

        json_data = external_theme.to_json()
        file.write(core.JsonFile.from_object(json_data).to_data())

    @staticmethod
    def parse_external_theme(path: core.Path) -> ExternalTheme | None:
        """Parses an external theme.

        Args:
            path (core.Path): Path to the external theme.

        Returns:
            ExternalTheme: External theme.
        """
        json_data = core.JsonFile.from_data(path.read()).to_object()
        return ExternalTheme.from_json(json_data)

    @staticmethod
    def update_external_theme(external_theme: ExternalTheme):
        """Updates an external theme.

        Args:
            external_theme (ExternalTheme): External theme to update.
        """
        if external_theme.git_repo is None:
            return
        color.ColoredText.localize(
            "checking_for_theme_updates",
            theme_name=external_theme.name,
        )
        updated = external_theme.get_new_version()
        if updated:
            color.ColoredText.localize(
                "external_theme_updated",
                theme_name=external_theme.name,
                version=external_theme.version,
            )
        else:
            color.ColoredText.localize(
                "external_theme_no_update",
                theme_name=external_theme.name,
                version=external_theme.version,
            )
        print()

    @staticmethod
    def update_all_external_themes(_: Any = None):
        """Updates all external themes."""
        files = ThemeHandler.get_external_themes_folder().get_paths_dir()
        if not files:
            color.ColoredText.localize(
                "no_external_themes",
            )
            return
        if not core.GitHandler.is_git_installed():
            color.ColoredText.localize(
                "git_not_installed",
            )
            return
        for file in files:
            theme = ExternalThemeManager.parse_external_theme(file)
            if theme is None:
                continue
            ExternalThemeManager.update_external_theme(theme)

    @staticmethod
    def get_external_theme_config() -> ExternalTheme | None:
        """Gets the external theme from the config.

        Returns:
            ExternalTheme: External theme.
        """

        theme = core.core_data.config.get_str(core.ConfigKey.THEME)
        if not theme.startswith("ext-"):
            return None
        return ExternalThemeManager.parse_external_theme(
            ThemeHandler.get_theme_path(theme)
        )

    @staticmethod
    def get_external_theme(theme: str) -> ExternalTheme | None:
        """Gets the external theme from the theme code.

        Returns:
            ExternalTheme: External theme.
        """

        if not theme.startswith("ext-"):
            return None
        return ExternalThemeManager.parse_external_theme(
            ThemeHandler.get_theme_path(theme)
        )



from __future__ import annotations
from typing import Any
from bcps import core


class BeaconEventListScene:
    def __init__(
        self,
        int_dict: dict[int, int],
        str_dict: dict[int, list[str]],
        bool_dict: dict[int, bool],
    ):
        self.int_array = int_dict
        self.str_array = str_dict
        self.bool_array = bool_dict

    @staticmethod
    def init() -> BeaconEventListScene:
        return BeaconEventListScene({}, {}, {})

    @staticmethod
    def read(stream: core.Data) -> BeaconEventListScene:
        int_dict = {}
        str_dict = {}
        bool_dict = {}
        for _ in range(stream.read_int()):
            int_dict[stream.read_int()] = stream.read_int()
        for _ in range(stream.read_int()):
            str_dict[stream.read_int()] = stream.read_string_list()
        for _ in range(stream.read_int()):
            bool_dict[stream.read_int()] = stream.read_bool()
        return BeaconEventListScene(int_dict, str_dict, bool_dict)

    def write(self, stream: core.Data):
        stream.write_int(len(self.int_array))
        for key, value in self.int_array.items():
            stream.write_int(key)
            stream.write_int(value)
        stream.write_int(len(self.str_array))
        for key, value in self.str_array.items():
            stream.write_int(key)
            stream.write_string_list(value)
        stream.write_int(len(self.bool_array))
        for key, value in self.bool_array.items():
            stream.write_int(key)
            stream.write_bool(value)

    def serialize(self) -> dict[str, Any]:
        return {
            "int_array": self.int_array,
            "str_array": self.str_array,
            "bool_array": self.bool_array,
        }

    @staticmethod
    def deserialize(data: dict[str, Any]) -> BeaconEventListScene:
        return BeaconEventListScene(
            data.get("int_array", []),
            data.get("str_array", []),
            data.get("bool_array", []),
        )

    def __repr__(self):
        return f"BeaconEventListScene({self.int_array}, {self.str_array}, {self.bool_array})"

    def __str__(self):
        return f"BeaconEventListScene({self.int_array}, {self.str_array}, {self.bool_array})"



from bcps.core.server import (
    managed_item,
    headers,
    client_info,
    server_handler,
    game_data_getter,
    request,
    updater,
)

__all__ = [
    "managed_item",
    "server_handler",
    "headers",
    "client_info",
    "game_data_getter",
    "request",
    "updater",
]



from __future__ import annotations
from typing import Any

from requests.exceptions import ConnectionError
from requests import Response
from json.decoder import JSONDecodeError
from bcps.cli import color

from bcps.core import (
    country_code,
    crypto,
    game,
    game_version,
    io,
    locale_handler,
    log,
    server,
    theme_handler,
    max_value_helper,
)
from bcps.core.country_code import CountryCode, CountryCodeType
from bcps.core.crypto import Hash, HashAlgorithm, Hmac, NyankoSignature, Random
from bcps.core.game.battle.battle_items import BattleItems
from bcps.core.game.battle.cleared_slots import ClearedSlots
from bcps.core.game.battle.slots import LineUps
from bcps.core.game.battle.enemy import Enemy, EnemyNames
from bcps.core.game.catbase.beacon_base import BeaconEventListScene
from bcps.core.game.catbase.cat import Cat, Cats, UnitBuy, TalentData
from bcps.core.game.catbase.gatya import Gatya, GatyaInfos, GatyaDataSet
from bcps.core.game.catbase.gatya_item import GatyaItemBuy, GatyaItemNames
from bcps.core.game.catbase.item_pack import (
    ItemPack,
    Purchases,
    PurchaseSet,
    PurchasedPack,
)
from bcps.core.game.catbase.login_bonuses import LoginBonus
from bcps.core.game.catbase.matatabi import Matatabi
from bcps.core.game.catbase.drop_chara import CharaDrop
from bcps.core.game.catbase.medals import Medals, MedalNames
from bcps.core.game.catbase.mission import Missions, MissionNames, MissionConditions
from bcps.core.game.catbase.my_sale import MySale
from bcps.core.game.catbase.nyanko_club import NyankoClub
from bcps.core.game.catbase.officer_pass import OfficerPass
from bcps.core.game.catbase.powerup import PowerUpHelper
from bcps.core.game.catbase.scheme_items import SchemeItems
from bcps.core.game.catbase.special_skill import (
    SpecialSkills,
    SpecialSkill,
    AbilityData,
    AbilityDataItem,
)
from bcps.core.game.catbase.stamp import StampData
from bcps.core.game.catbase.talent_orbs import (
    TalentOrb,
    TalentOrbs,
    OrbInfo,
    OrbInfoList,
    RawOrbInfo,
    SaveOrb,
    SaveOrbs,
)
from bcps.core.game.catbase.unlock_popups import UnlockPopups
from bcps.core.game.catbase.upgrade import Upgrade
from bcps.core.game.catbase.user_rank_rewards import (
    UserRankRewards,
    RankGifts,
    RankGiftDescriptions,
)
from bcps.core.game.catbase.playtime import PlayTime
from bcps.core.game.gamoto.base_materials import BaseMaterials
from bcps.core.game.gamoto.cat_shrine import CatShrine, CatShrineLevels
from bcps.core.game.gamoto.gamatoto import (
    Gamatoto,
    GamatotoLevels,
    GamatotoMembersName,
)
from bcps.core.game.gamoto.ototo import Ototo
from bcps.core.game.localizable import Localizable
from bcps.core.game.map.aku import AkuChapters
from bcps.core.game.map.challenge import ChallengeChapters
from bcps.core.game.map.chapters import Chapters
from bcps.core.game.map.dojo import Dojo
from bcps.core.game.map.enigma import Enigma
from bcps.core.game.map.event import EventChapters
from bcps.core.game.map.ex_stage import ExChapters
from bcps.core.game.map.gauntlets import GauntletChapters
from bcps.core.game.map.item_reward_stage import ItemRewardChapters
from bcps.core.game.map.legend_quest import LegendQuestChapters
from bcps.core.game.map.map_reset import MapResets
from bcps.core.game.map.outbreaks import Outbreaks
from bcps.core.game.map.story import StoryChapters, TreasureText, StageNames
from bcps.core.game.map.timed_score import TimedScoreChapters
from bcps.core.game.map.tower import TowerChapters
from bcps.core.game.map.uncanny import UncannyChapters
from bcps.core.game.map.zero_legends import ZeroLegendsChapters
from bcps.core.game.map.map_names import MapNames
from bcps.core.game_version import GameVersion
from bcps.core.io.adb_handler import AdbHandler, AdbNotInstalled
from bcps.core.io.bc_csv import CSV, Delimeter, Row
from bcps.core.io.command import Command, CommandResult
from bcps.core.io.config import Config, ConfigKey
from bcps.core.io.data import Data
from bcps.core.io.json_file import JsonFile
from bcps.core.io.path import Path
from bcps.core.io.save import SaveError, SaveFile, CantDetectSaveCCError
from bcps.core.io.thread_helper import thread_run_many, Thread
from bcps.core.io.yaml import YamlFile
from bcps.core.io.git_handler import GitHandler, Repo
from bcps.core.io.root_handler import RootHandler
from bcps.core.locale_handler import (
    LocalManager,
    ExternalLocaleManager,
    ExternalLocale,
)
from bcps.core.log import Logger
from bcps.core.server.client_info import ClientInfo
from bcps.core.server.game_data_getter import GameDataGetter
from bcps.core.server.headers import AccountHeaders
from bcps.core.server.managed_item import BackupMetaData, ManagedItem, ManagedItemType
from bcps.core.server.request import RequestHandler
from bcps.core.server.server_handler import ServerHandler
from bcps.core.server.updater import Updater
from bcps.core.theme_handler import ThemeHandler, ExternalTheme, ExternalThemeManager
from bcps.core.max_value_helper import MaxValueHelper


class CoreData:
    def init_data(self):
        self.config = Config()
        self.logger = Logger()
        self.local_manager = LocalManager()
        self.theme_manager = ThemeHandler()
        self.max_value_manager = MaxValueHelper()
        self.game_data_getter: GameDataGetter | None = None
        self.gatya_item_names: GatyaItemNames | None = None
        self.gatya_item_buy: GatyaItemBuy | None = None
        self.chara_drop: CharaDrop | None = None
        self.gamatoto_levels: GamatotoLevels | None = None
        self.gamatoto_members_name: GamatotoMembersName | None = None
        self.localizable: Localizable | None = None
        self.abilty_data: AbilityData | None = None
        self.enemy_names: EnemyNames | None = None
        self.rank_gift_descriptions: RankGiftDescriptions | None = None
        self.rank_gifts: RankGifts | None = None
        self.treasure_text: TreasureText | None = None
        self.cat_shrine_levels: CatShrineLevels | None = None
        self.medal_names: MedalNames | None = None
        self.mission_names: MissionNames | None = None
        self.mission_conditions: MissionConditions | None = None

    def get_game_data_getter(
        self, save: SaveFile | None = None, cc: CountryCode | None = None
    ) -> GameDataGetter:
        if self.game_data_getter is None:
            if cc is None and save is not None:
                cc = save.cc
            if cc is None:
                raise ValueError("cc must be provided if save is not provided")
            self.game_data_getter = GameDataGetter(cc)
        return self.game_data_getter

    def get_gatya_item_names(self, save: SaveFile) -> GatyaItemNames:
        if self.gatya_item_names is None:
            self.gatya_item_names = GatyaItemNames(save)
        return self.gatya_item_names

    def get_gatya_item_buy(self, save: SaveFile) -> GatyaItemBuy:
        if self.gatya_item_buy is None:
            self.gatya_item_buy = GatyaItemBuy(save)
        return self.gatya_item_buy

    def get_chara_drop(self, save: SaveFile) -> CharaDrop:
        if self.chara_drop is None:
            self.chara_drop = CharaDrop(save)
        return self.chara_drop

    def get_gamatoto_levels(self, save: SaveFile) -> GamatotoLevels:
        if self.gamatoto_levels is None:
            self.gamatoto_levels = GamatotoLevels(save)
        return self.gamatoto_levels

    def get_gamatoto_members_name(self, save: SaveFile) -> GamatotoMembersName:
        if self.gamatoto_members_name is None:
            self.gamatoto_members_name = GamatotoMembersName(save)
        return self.gamatoto_members_name

    def get_localizable(self, save: SaveFile) -> Localizable:
        if self.localizable is None:
            self.localizable = Localizable(save)
        return self.localizable

    def get_ability_data(self, save: SaveFile) -> AbilityData:
        if self.abilty_data is None:
            self.abilty_data = AbilityData(save)
        return self.abilty_data

    def get_enemy_names(self, save: SaveFile) -> EnemyNames:
        if self.enemy_names is None:
            self.enemy_names = EnemyNames(save)
        return self.enemy_names

    def get_rank_gift_descriptions(self, save: SaveFile) -> RankGiftDescriptions:
        if self.rank_gift_descriptions is None:
            self.rank_gift_descriptions = RankGiftDescriptions(save)
        return self.rank_gift_descriptions

    def get_rank_gifts(self, save: SaveFile) -> RankGifts:
        if self.rank_gifts is None:
            self.rank_gifts = RankGifts(save)
        return self.rank_gifts

    def get_treasure_text(self, save: SaveFile) -> TreasureText:
        if self.treasure_text is None:
            self.treasure_text = TreasureText(save)
        return self.treasure_text

    def get_cat_shrine_levels(self, save: SaveFile) -> CatShrineLevels:
        if self.cat_shrine_levels is None:
            self.cat_shrine_levels = CatShrineLevels(save)
        return self.cat_shrine_levels

    def get_medal_names(self, save: SaveFile) -> MedalNames:
        if self.medal_names is None:
            self.medal_names = MedalNames(save)
        return self.medal_names

    def get_mission_names(self, save: SaveFile) -> MissionNames:
        if self.mission_names is None:
            self.mission_names = MissionNames(save)
        return self.mission_names

    def get_mission_conditions(self, save: SaveFile) -> MissionConditions:
        if self.mission_conditions is None:
            self.mission_conditions = MissionConditions(save)
        return self.mission_conditions

    def get_lang(self, save: SaveFile) -> str:
        return self.get_localizable(save).get_lang() or "en"


def update_external_content(_: Any = None):
    """Updates external content."""

    color.ColoredText.localize("updating_external_content")
    print()
    ExternalThemeManager.update_all_external_themes()
    ExternalLocaleManager.update_all_external_locales()
    core_data.init_data()


def print_no_internet():
    color.ColoredText.localize("no_internet")


core_data = CoreData()
core_data.init_data()


__all__ = [
    "server",
    "io",
    "locale_handler",
    "country_code",
    "log",
    "game_version",
    "crypto",
    "game",
    "theme_handler",
    "max_value_helper",
    "AdbHandler",
    "AdbNotInstalled",
    "CountryCode",
    "Path",
    "Data",
    "CSV",
    "ServerHandler",
    "GameVersion",
    "SaveFile",
    "JsonFile",
    "ManagedItem",
    "ManagedItemType",
    "BackupMetaData",
    "Cat",
    "Upgrade",
    "PowerUpHelper",
    "TalentOrb",
    "TalentOrbs",
    "OrbInfo",
    "OrbInfoList",
    "RawOrbInfo",
    "SaveOrb",
    "SaveOrbs",
    "ConfigKey",
    "SpecialSkill",
]
